@@ -4,5 +4,5 @@ from django.db import models
 
 
 class Analytic(models.Model):
-    site = models.ForeignKey(Site, unique=True)
+    site = models.OneToOneField(Site)
     analytics_code = models.CharField(blank=True, max_length=100)
