@@ -51,6 +51,8 @@ from lava_results_app.views.query.views import (
     query_group_list,
     query_refresh,
     query_remove_condition,
+    query_omit_result,
+    query_include_result,
     query_select_group,
     query_toggle_published,
 )
@@ -71,6 +73,7 @@ from lava_results_app.views.chart.views import (
     chart_query_edit,
     chart_query_remove,
     chart_query_order_update,
+    chart_omit_result,
     settings_update
 )
 
@@ -102,6 +105,8 @@ urlpatterns = [
         name='lava.results.query_edit_condition'),
     url(r'^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+add-group$', query_add_group, name='query_add_group'),
     url(r'^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/\+select-group$', query_select_group, name='query_select_group'),
+    url(r'^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+omit-result$', query_omit_result, name='lava.results.query_omit_result'),
+    url(r'^query/~(?P<username>[^/]+)/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+include-result$', query_include_result, name='lava.results.query_include_result'),
     url(r'^query/get-query-groups$', query_group_list, name='query_group_list'),
     url(r'^query/\+get-group-names$', get_query_group_names),
     url(r'^query/\+get-query-names$', get_query_names),
@@ -137,6 +142,7 @@ urlpatterns = [
     url(r'^chart/\+get-group-names$', get_chart_group_names),
     url(r'^chart/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/\+settings-update$', settings_update, name='chart_settings_update'),
     url(r'^chart/(?P<name>[a-zA-Z0-9-_]+)/\+chart-query-order-update$', chart_query_order_update, name='chart_query_order_update'),
+    url(r'^chart/(?P<name>[a-zA-Z0-9-_]+)/(?P<id>\d+)/(?P<result_id>\d+)/\+omit-result$', chart_omit_result, name='lava.results.chart_omit_result'),
     url(r'^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/(?P<ts>[-_a-zA-Z0-9.]+)/(?P<case>[-_a-zA-Z0-9.]+)$', testset, name='lava.results.testset'),
     url(r'^(?P<job>[0-9]+|[0-9]+.[0-9]+)/(?P<pk>[-_a-zA-Z0-9.]+)/csv$',
         suite_csv, name='lava.results.suite_csv'),
