@@ -19,31 +19,27 @@
 import csv
 import json
 import os
-import psycopg2
 import shutil
 import simplejson
 import tempfile
 
+from django.db import IntegrityError
 from django.db.utils import ProgrammingError
 from django.contrib.auth.decorators import login_required
-from django.contrib.auth.models import Group, User
+from django.contrib.auth.models import Group
 from django.core import serializers
 from django.core.exceptions import (
     PermissionDenied,
-    ValidationError,
     FieldError
 )
 from django.core.urlresolvers import reverse
 from django.http import (
     HttpResponse,
-    HttpResponseBadRequest,
     HttpResponseRedirect
 )
 from django.shortcuts import get_object_or_404, render_to_response
 from django.template import RequestContext, defaultfilters
-from django.utils.safestring import mark_safe
 
-from dashboard_app.models import NamedAttribute
 from lava_server.bread_crumbs import (
     BreadCrumb,
     BreadCrumbTrail,
@@ -60,6 +56,7 @@ from lava_results_app.models import (
     Query,
     QueryGroup,
     QueryCondition,
+    QueryOmitResult,
     QueryMaterializedView,
     QueryUpdatedError,
     TestCase,
@@ -217,7 +214,7 @@ def query_list(request):
             'terms_data': terms_data,
             'group_tables': group_tables,
             'bread_crumb_trail': BreadCrumbTrail.leading_to(query_list),
-            'context_help': BreadCrumbTrail.leading_to(query_list),
+            'context_help': ['lava-queries-charts'],
         }, RequestContext(request)
     )
 
@@ -228,6 +225,10 @@ def query_display(request, username, name):
 
     query = get_object_or_404(Query, owner__username=username, name=name)
 
+    if not request.user.is_superuser:
+        if not query.is_published and query.owner != request.user:
+            raise PermissionDenied
+
     view = QueryResultView(
         query=query,
         request=request,
@@ -236,6 +237,8 @@ def query_display(request, username, name):
     )
 
     table = QUERY_CONTENT_TYPE_TABLE[query.content_type.model_class()](
+        query,
+        request.user,
         view.get_table_data()
     )
 
@@ -247,18 +250,21 @@ def query_display(request, username, name):
             "Query view does not exist. Please contact query owner or system "
             "administrator.")
 
+    omitted = [result.content_object for result in QueryOmitResult.objects.filter(query=query)]
+
     return render_to_response(
         'lava_results_app/query_display.html', {
             'query': query,
             'entity': query.content_type.model,
             'conditions': query.serialize_conditions(),
+            'omitted': omitted,
             'query_table': table,
             'terms_data': table.prepare_terms_data(view),
             'search_data': table.prepare_search_data(view),
             'discrete_data': table.prepare_discrete_data(view),
             'bread_crumb_trail': BreadCrumbTrail.leading_to(
                 query_display, username=username, name=name),
-            'context_help': BreadCrumbTrail.leading_to(query_list),
+            'context_help': ['lava-queries-charts'],
         }, RequestContext(request)
     )
 
@@ -284,6 +290,8 @@ def query_custom(request):
 
     try:
         table = QUERY_CONTENT_TYPE_TABLE[content_type.model_class()](
+            None,
+            request.user,
             view.get_table_data()
         )
     except FieldError:
@@ -302,7 +310,7 @@ def query_custom(request):
             'discrete_data': table.prepare_discrete_data(view),
 
             'bread_crumb_trail': BreadCrumbTrail.leading_to(query_custom),
-            'context_help': BreadCrumbTrail.leading_to(query_list),
+            'context_help': ['lava-queries-charts'],
         }, RequestContext(request)
     )
 
@@ -324,7 +332,7 @@ def query_detail(request, username, name):
             'view_exists': view_exists,
             'bread_crumb_trail': BreadCrumbTrail.leading_to(
                 query_detail, username=username, name=name),
-            'context_help': BreadCrumbTrail.leading_to(query_list),
+            'context_help': ['lava-queries-charts'],
             'condition_form': QueryConditionForm(
                 instance=None,
                 initial={'query': query, 'table': query.content_type}),
@@ -448,6 +456,7 @@ def query_copy(request, username, name):
 
 
 @login_required
+@ownership_required
 def query_refresh(request, name, username):
 
     query = get_object_or_404(Query, owner__username=username, name=name)
@@ -574,6 +583,42 @@ def query_remove_condition(request, username, name, id):
     return HttpResponseRedirect(query.get_absolute_url() + "/+detail")
 
 
+@login_required
+@ownership_required
+def query_omit_result(request, username, name, id):
+
+    query = get_object_or_404(Query, owner__username=username, name=name)
+    result_object = get_object_or_404(query.content_type.model_class(), id=id)
+
+    try:
+        QueryOmitResult.objects.create(query=query,
+                                       content_object=result_object)
+    except IntegrityError:
+        # Ignore unique constraint violation.
+        pass
+
+    return HttpResponseRedirect(query.get_absolute_url())
+
+
+@login_required
+@ownership_required
+def query_include_result(request, username, name, id):
+
+    query = get_object_or_404(Query, owner__username=username, name=name)
+    result_object = get_object_or_404(query.content_type.model_class(), id=id)
+
+    try:
+        QueryOmitResult.objects.get(
+            query=query,
+            object_id=result_object.id,
+            content_type=query.content_type).delete()
+    except QueryOmitResult.DoesNotExist:
+        # Ignore does not exist violation.
+        raise
+
+    return HttpResponseRedirect(query.get_absolute_url())
+
+
 def get_query_names(request):
 
     term = request.GET['term']
@@ -622,6 +667,7 @@ def query_form(request, bread_crumb_trail, instance=None, is_copy=False):
             'is_copy': is_copy,
             'query_name': query_name,
             'form': form,
+            'context_help': ['lava-queries-charts'],
         }, RequestContext(request))
 
 
@@ -647,7 +693,7 @@ def _remove_dir(path):
     try:
         # Delete directory.
         shutil.rmtree(path)
-    except OSError as exception:
+    except OSError:
         # Silent exception whatever happens. If it's unexisting dir, we don't
         # care.
         pass
