@@ -20,6 +20,7 @@
 import os
 import re
 import copy
+import errno
 import yaml
 import pprint
 import jinja2
@@ -40,6 +41,7 @@ from django.utils import timezone
 
 from lava_server.settings.getsettings import Settings
 from lava_server.settings.config_file import ConfigFile
+from lava_scheduler_app.schema import SubmissionException
 
 
 def get_fqdn():
@@ -88,6 +90,7 @@ def split_multi_job(json_jobdata, target_group):
     all_nodes = {}
     node_actions = {}
     node_lmp = {}
+    shared_config = get_shared_device_config("/etc/lava-server/shared-device-config.yaml")
 
     # Check if we are operating on multinode job data. Else return the job
     # data as it is.
@@ -161,6 +164,8 @@ def split_multi_job(json_jobdata, target_group):
             if json_jobdata.get("priority", False):
                 node_json[role][c]["priority"] = json_jobdata["priority"]
             node_json[role][c]["device_type"] = clients["device_type"]
+            if shared_config:
+                node_json[role][c]["shared_config"] = shared_config
 
     return node_json
 
@@ -732,7 +737,7 @@ def _split_multinode_vland(submission, jobs):
     for role, _ in jobs.iteritems():
         # populate the lava-vland protocol metadata
         if len(jobs[role]) != 1:
-            raise models.SubmissionException("vland protocol only supports one device per role.")
+            raise SubmissionException("vland protocol only supports one device per role.")
         jobs[role][0]['protocols'].update({'lava-vland': submission['protocols']['lava-vland'][role]})
     return jobs
 
@@ -809,7 +814,7 @@ def split_multinode_yaml(submission, target_group):  # pylint: disable=too-many-
                 try:
                     value['role']
                 except (KeyError, TypeError):
-                    raise models.SubmissionException("Invalid YAML - check for consistent use of whitespace indents.")
+                    raise SubmissionException("Invalid YAML - check for consistent use of whitespace indents.")
                 if role in value['role']:
                     actions.setdefault(role, {'actions': []})
                     actions[role]['actions'].append({copy.deepcopy(key): copy.deepcopy(value)})
@@ -838,7 +843,7 @@ def split_multinode_yaml(submission, target_group):  # pylint: disable=too-many-
     for role in roles:
         if role == check_count:
             if roles[role]['count'] != 1:
-                raise models.SubmissionException('The count for a role designated as a host_role must be 1.')
+                raise SubmissionException('The count for a role designated as a host_role must be 1.')
     for role in roles:
         jobs[role] = []
         for sub in range(0, roles[role]['count']):
@@ -860,3 +865,26 @@ def split_multinode_yaml(submission, target_group):  # pylint: disable=too-many-
     if 'lava-vland' in submission['protocols']:
         _split_multinode_vland(submission, jobs)
     return jobs
+
+
+def get_shared_device_config(filename):
+    config_dict = {}
+    if os.path.isfile(filename):
+        try:
+            with open(filename, 'r') as f:
+                config_dict = yaml.load(f.read())
+        except (yaml.YAMLError, IOError):
+            return None
+    else:
+        return None
+    return config_dict
+
+
+def mkdir(path):
+    try:
+        os.makedirs(path)
+    except OSError as exc:
+        if exc.errno == errno.EEXIST and os.path.isdir(path):
+            pass
+        else:
+            raise
