@@ -1,9 +1,9 @@
-#!/bin/bash
+#!/bin/sh
 
 set -e
 
 BRANCH=master
-arch=''
+ARCH=''
 
 while getopts ":p:a:b:" opt; do
   case $opt in
@@ -11,7 +11,15 @@ while getopts ":p:a:b:" opt; do
       NAME=$OPTARG
       ;;
     a)
-      arch=$OPTARG
+      ARCH=$OPTARG
+      set +e
+      chk=`dpkg-architecture -a$ARCH > /dev/null 2>&1 ; echo $?`
+      set -e
+      if [ $chk != 0 ]; then
+          echo "Did not recognise ${ARCH} as a Debian architecture name. Exit."
+          exit 1
+      fi
+      ARCH="-a${ARCH} -b"
       ;;
     b)
       BRANCH=$OPTARG
@@ -24,29 +32,12 @@ done
 
 if [ -z "$NAME" ]; then
 
-    if [ -z "$1" ]; then
-        echo "Usage: <package> [<architecture>]"
-        echo "If architecture is a known Debian architecture, build"
-        echo "a binary-only package for this architecture."
-        echo "e.g. armhf or arm64"
-        exit 1
-    fi
-
-    if [ -n "$2" ]; then
-        set +e
-        chk=`dpkg-architecture -a$2 > /dev/null 2>&1 ; echo $?`
-        set -e
-        if [ $chk = 0 ]; then
-            echo "Building for architecture $2"
-            arch="-a$2 -b"
-        else
-            echo "Did not recognise $2 as a Debian architecture name. Exit."
-            exit 1
-        fi
-    fi
-    PWD=`pwd`
-    NAME=${1}
-    BRANCH=master
+    echo "Usage: -p <package> [-a <architecture> -b <branch>]"
+    echo "Builds a sourceful package locally, using debuild."
+    echo "If architecture is a known Debian architecture, build"
+    echo "a binary-only package for this architecture. e.g. armhf or arm64"
+    echo "Branch specifies the packaging branch to use from github."
+    exit 1
 fi
 if [ -x ./version.py ]; then
   VERSION=`python ./version.py`
@@ -66,7 +57,7 @@ if [ ! -e ./dist/${NAME}-${VERSION}.tar.gz ]; then
 	VERSION=`echo ${VERSION}| sed -e 's/\([0-9]\)+/\1-/'`
 fi
 DIR=`mktemp -d`
-if [ -f './dist/${NAME}-${VERSION}.tar.gz' ]; then
+if [ -f "./dist/${NAME}-${VERSION}.tar.gz" ]; then
   mv -v ./dist/${NAME}-${VERSION}.tar.gz ${DIR}/${NAME}_${VERSION}.orig.tar.gz
 else
   echo "WARNING: broken setuptools tarball - Debian bug #786977"
@@ -87,7 +78,7 @@ dch -v ${VERSION}-1 -D unstable "Local developer build"
 if [ -n "${LOG}" ]; then
   dch -a "${LOG}"
 fi
-debuild -sa -uc -us $arch
+debuild -sa -uc -us $ARCH
 cd ${DIR}
 rm -rf ${DIR}/pkg-${NAME}
 rm -rf ${DIR}/${NAME}-${VERSION}
